////////////////////////////////////////////////////////
//	HEPEvtLiteUtils.h
//	Author: V.Makarenko (Vladimir.Makarenko@cern.ch)
////////////////////////////////////////////////////////

#if !defined(HEPEvtLiteUtils_H_INCLUDED_)
#define HEPEvtLiteUtils_H_INCLUDED_
/*
*	Note: don't include this file directly
*	Pre-includes:
*	#include <iostream>
*	#include <fstream>
*	#include "VMUtil.h"
*	#include "VMOutput.h"
*	#include "VMHistLite.h"
*	#include "VMGen.h"
*	#include "VMHepEvtLite.h"
*/

//
//	CHEPEvtLiteLHE: interface for LHE file format
//
class CHEPEvtLiteLHE {
public:
	class LHEInfo {
	public:
		class LHEProc {
		public:
			tReal xSec;		//XSECUP(IPR)
			tReal xSecErr;	//XERRUP(IPR)
			tReal xMax;		//XMAXUP(IPR)
			int iProc;		//LPRUP(IPR)
			LHEProc() { xSec=0.; xSecErr=0.; xMax=0.; iProc=-1; }
			LHEProc(const LHEProc& p) { xSec=p.xSec; xSecErr=p.xSecErr; xMax=p.xMax; iProc=p.iProc; }
			LHEProc& operator=(const LHEProc& p) { xSec=p.xSec; xSecErr=p.xSecErr; xMax=p.xMax; iProc=p.iProc; return *this; }
			//LHEProc(tReal _xSec, tReal _xSecErr, tReal _xMax, int _iProc) { xSec=_xSec; xSecErr=_xSecErr; xMax=_xMax; iProc=_iProc; }
		};
	public:
		int idIn[2];	//IDBMUP(1),IDBMUP(2)
		tReal EIn[2];	//EBMUP(1), EBMUP(2)
		long idPDF[2];	//PDFGUP(1) PDFGUP(2)
		long subPDF[2];	//PDFSUP(1) PDFSUP(2)
		int idWgt;		//IDWTUP
		int nProcs;		//NPRUP
	private:
		std::vector< LHEProc* > procs;
	public:
		LHEInfo(int _idIn1, int _idIn2,
			tReal _EIn1, tReal _EIn2,
			long _idPDF1, long _subPDF1, long _idPDF2, long _subPDF2,
			int _idWgt, int _nProcs);
		~LHEInfo();
		void AddProc(const LHEProc& p);
		void AddProc(const CHEPProcessLite* proc);
		const LHEProc* GetProcByID(int id);
		const LHEProc* GetProc(int i);
	};

private:
	char* m_file_name;							//NULL if not ever open yet
	bool m_bReadMode;							//otherwise write
	LHEInfo* m_info;

public:
	CHEPEvtLiteLHE();
	CHEPEvtLiteLHE(int _idIn1, int _idIn2,
					tReal _EIn1, tReal _EIn2,
					long _idPDF1, long _subPDF1, long _idPDF2, long _subPDF2,
					int _idWgt, int _nProcs);
	virtual ~CHEPEvtLiteLHE();
	void SetProcessInfo(const CHEPProcessLite* proc);

public:
	// --- Write mode ---
	bool StartLHEFile(const char* fn);
	bool SaveEvent(const CHEPEvtLite* evt);
	bool CloseLHEFile();						//not required for 'read' mode

	// --- Read mode ---
	bool OpenFile(const char* fn);
	bool LoadEvent(CHEPEvtLite* evt);
};

#endif // !defined(HEPEvtLiteUtils_H_INCLUDED_)
