////////////////////////////////////////////////////////
//	VMUtil.h
//	Author: V.Makarenko (Vladimir.Makarenko@cern.ch)
////////////////////////////////////////////////////////

#if !defined(VMUtil_H_INCLUDED_)
#define VMUtil_H_INCLUDED_

///////////////////////////////////////////// Utilites
class CVMUtil {
public:
	//library functions
	static bool strEqu(const char* a, const char* b);
	static long strLen(const char* a);
	static char* strCpy(const char* a);
	static char* strAlloc(long s);		//used to avoid 'malloc() or new char[]?' question
	static void strFree(char* a);		//used to avoid 'free() or delete[]?' question
	//
	static bool bIsSymbol(char c);
	static bool bIsNumber(char c);
	//
	static void Cout(std::ostream* os, tReal val);	//auto-select form ('0.000234' or '2.34^-10')
};

class CVMFileIn { //used to save lines for stream operations
	std::ifstream m_ifs;
	bool m_bBinary;
	bool m_bReadOK;
public:
	bool m_bBinaryAsDouble; // Load 'tReal' numbers as 'double' in binary mode
public:
	CVMFileIn(const char* fn, bool bBinary=false);
	virtual ~CVMFileIn();
	//
	CVMFileIn& operator>>(char& c);
	CVMFileIn& operator>>(long& val);
	CVMFileIn& operator>>(unsigned long& val);
	CVMFileIn& operator>>(int& val);
	CVMFileIn& operator>>(unsigned int& val);
	CVMFileIn& operator>>(double& val);
#ifndef tReal_EQU_double
	CVMFileIn& operator>>(tReal& val); //Additional operation is 'tReal' != 'double'
#endif //tReal_EQU_double
	bool ReadReal(tReal& val, bool bBinary);
	bool ReadString(char* buf, long szBuf, const char* endMark=NULL); //skips 'endMark'
	bool ReadNotEmptySymbol(char& c); //skips spaces, tabs and line breaks
public:
	bool ReadOK() { return m_bReadOK; }
	bool IsReady() { return (m_ifs.is_open() && m_ifs.good()); }
	bool IsBinary() { return m_bBinary; }
	std::ifstream* GetStream() { return &m_ifs; }
};
class CVMFileOut { //used to save lines for stream operations
	char* m_filename;
	std::ofstream* m_ofs;
	bool m_bKeepOpened;
	bool m_bBinary;
	bool PrepareStream(); //creates stream if !m_bKeepOpened, m_ofs!=NULL if result is 'true'
	void ReleaseStream(); //clears stream if !m_bKeepOpened
public:
	bool m_bBinaryAsDouble; // Save 'tReal' numbers as 'double' in binary mode
public:
	CVMFileOut(const char* fn, bool bKeepOpened, bool bBinary=false);
	virtual ~CVMFileOut();
	std::ofstream* GetStream() { return m_ofs; } //for external calls only
	bool Open();
	void Close();
	bool IsBinary() { return m_bBinary; }
	bool IsReady();

	//Nt: every operation opens+closes file if not open
	bool WriteReal(tReal val, bool bBinary);
	bool WriteBuffer(const char* buf);
	CVMFileOut& operator<<(long val);
	//CVMFileOut& operator<<(unsigned long val);
	CVMFileOut& operator<<(double val);
#ifndef tReal_EQU_double
	CVMFileOut& operator<<(tReal val); //Additional operation is 'tReal' != 'double'
#endif //tReal_EQU_double
	CVMFileOut& operator<<(const char* str);
};

#endif // !defined(VMUtil_H_INCLUDED_)
