////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// vmlib.h: headers for vmlib library
//
//////////////////////////////////////////////////////////////////////

#if !defined(VMLIB_H_INCLUDED_)
#define VMLIB_H_INCLUDED_

#include <iostream>
#include <fstream>
#include <vector>

//platform flag init: 'VMLIB_WIN' for Win32, otherwise for Linux/Unix...
#if defined _WIN32
#define VMLIB_WIN
#elif defined _WIN64
#define VMLIB_WIN
#elif defined WIN32
#define VMLIB_WIN
#elif defined WIN64
#define VMLIB_WIN
#endif

////////////////////////////////////////////////////////////////////// library options BEGIN
/*
*	LoopTools usage flags
*/
#ifdef VMLIB_WIN
//Win-specific definitions (debug)
#define BH_NO_LOOPS
//#define VM_DEBUG_FULL
#endif //VMLIB_WIN
#ifndef BH_NO_LOOPS
#define VM_USE_LOOPTOOLS
//#define VM_LOOPTOOLS_QUAD
#endif //BH_NO_LOOPS

/*
*	Pythia8 usage
*		Default: not used
*		Note: this flag is typically set in makefile by -D'VM_USE_PYTHIA8'
*	For Pythia xml-path the environment variable 'PYTHIA8DATA' is used
*/
#define VM_USE_PYTHIA8
//
//
#ifdef VMLIB_WIN
//	Set Pythia path manually for Win32
#define VM_PYTHIA8_XML_PATH		"D:/Work/Projects/Pythia8/xmldoc/"
#else //#ifdef VMLIB_WIN
//	Define nothing -- 'PYTHIA8DATA' is used
#endif //#ifdef VMLIB_WIN

#ifdef VM_USE_PYTHIA8
//const char* py_path = "D:/Work/Projects/Pythia8/xmldoc/"; //xmldoc/Index.xml
#endif //#ifdef VM_USE_PYTHIA8

/*
*	High-precision variables usage:
*		pre-define 'BHCODE_8', 'BHCODE_16' or 'BHCODE_32' or modify default settings below
*/
#if defined BHCODE_8
#elif defined BHCODE_16
#define VM_USE_DD_REAL
#elif defined BHCODE_32
#define VM_USE_QD_REAL
#else
#ifndef VMLIB_WIN
// Linux/unix default precision
// Uncomment ONE of the lines below to use high-precision variables
//#define VM_USE_QD_REAL
//#define VM_USE_DD_REAL
//#define VM_USE_LONG_DOUBLE
#else
// Windows default precision (no 'VM_USE_LONG_DOUBLE'!)
// Uncomment ONE of the lines below to use high-precision variables
//#define VM_USE_QD_REAL
//#define VM_USE_DD_REAL
#endif //#ifndef VMLIB_WIN
#endif

/*
*  Library content flags
*    'VM_HEP_LIB_FULL' includes additional JesReco-lib, Histos and Tests
*    (Not recommended! The standard procedures are more reliable!)
*/
#define VM_HEP_LIB_FULL

/*
*	Deprecated options
*/
// CVM_1VER_ONLY_ADV: used by bh-project
#define CVM_1VER_ONLY_ADV
////////////////////////////////////////////////////////////////////// library options END

////////////////////////////////////////////////////////////////////// tReal selection BEGIN
//	Do not modify this section
//
/*
*	The ID of current type is defined as 'VM_TREAL_ID'
*	Possible values: VM_TREAL_XXX
*	Warning: don't change the numbers!
*/
#define VM_TREAL_ERROR		0
#define VM_TREAL_DBL		1
#define VM_TREAL_LONGDBL	2
#define VM_TREAL_DD			3
#define VM_TREAL_QD			4

#ifdef VM_USE_QD_REAL
#ifdef VM_USE_DD_REAL
#undef VM_USE_DD_REAL
#endif //VM_USE_DD_REAL
#define VM_TREAL_ID	VM_TREAL_QD
#include "qd/qd_real.h"
#define tReal qd_real
#define tReal_DEFINED
//Use local 'qd_real_DEFINED' instead of '_QD_QD_REAL_H'
#define qd_real_DEFINED
#define dd_real_DEFINED
#define DBLtR(x) to_double(x)
//TODO: make to_double(qd_real) inline -- see implementation of qd_real
inline double to_double(const double &a){ return a; } //to extra (double)(tReal)(double x) conversion in DBLtR(double)
#define epsReal	qd_real::_eps
//Note: pow(int,int) must be casted manually in case of 'qd_real' type is used
#ifdef VMLIB_WIN
//VC compiller have pow(double,int) function
#define vmPow(x,y) pow((double)x,y)
#else
//gcc: pow(double,double) only
#define vmPow(x,y) pow((double)x,(double)y)
#endif //VMLIB_WIN
#endif //VM_USE_QD_REAL

#ifndef tReal_DEFINED
#ifdef VM_USE_DD_REAL
#define VM_TREAL_ID	VM_TREAL_DD
#include "qd/dd_real.h"
#define tReal dd_real
#define tReal_DEFINED
//Use local 'dd_real_DEFINED' instead of '_QD_DD_REAL_H'
#define dd_real_DEFINED
#define DBLtR(x) to_double(x)
//TODO: make to_double(dd_real) inline -- see implementation of dd_real
inline double to_double(const double &a){ return a; } //to extra (double)(tReal)(double x) conversion in DBLtR(double)
#define epsReal	dd_real::_eps
//Note: pow(int,int) must be casted manually in case of 'dd_real' type is used
#ifdef VMLIB_WIN
//VC compiller have pow(double,int) function
#define vmPow(x,y) pow((double)x,y)
#else
//gcc: pow(double,double) only
#define vmPow(x,y) pow((double)x,(double)y)
#endif //VMLIB_WIN
#endif //VM_USE_DD_REAL
#endif //tReal_DEFINED

#ifndef tReal_DEFINED
#ifdef VM_USE_LONG_DOUBLE
#define VM_TREAL_ID	VM_TREAL_LONGDBL
#define tReal long double
#define tReal_DEFINED
#define DBLtR(x) (double)(x)
//TODO insert SYSTEM 'eps' here!!!
#define epsReal	2.220446049250313e-16
#define vmPow(x,y) pow((long double)x,y)
#endif //VM_USE_LONG_DOUBLE
#endif //tReal_DEFINED

#ifndef tReal_DEFINED
#define VM_TREAL_ID VM_TREAL_DBL
#define tReal double
#define tReal_DEFINED
#define tReal_EQU_double
#define DBLtR(x) (double)(x)
#define epsReal	2.220446049250313e-16
#define vmPow(x,y) pow((double)x,y)
#endif //tReal_DEFINED

#ifndef VM_TREAL_ID
//should never occur
#define VM_TREAL_ID	VM_TREAL_ERROR
#endif //VM_TREAL_ID
////////////////////////////////////////////////////////////////////// tReal selection END

//Note: vmdecl is declared in "VMGen.h"

////////////////////////////////////////////////////////////////////// includes BEGIN
//	Here may be modified:
//		- path to PYTHIA includes
//

#ifdef VM_USE_PYTHIA8
//	Modity path here if required
//	Note: we don't put Pythia include-dir to common path
//	      because it contains regular-named files like 'Info.h' or 'Settings.h'
//
#include "Pythia8/Include/Pythia.h"
#endif //#ifdef VM_USE_PYTHIA8

#include "VMUtil.h"

#ifdef VM_HEP_LIB_FULL
#include "VMOutput.h"
#include "VMHistLite.h"
#endif //#ifdef VM_HEP_LIB_FULL

#include "VMGen.h"
#include "HEPEvtLite.h"
#include "HEPEvtLiteUtils.h"

#ifdef VM_HEP_LIB_FULL
#include "VMJetReco.h"
#endif //#ifdef VM_HEP_LIB_FULL

#include "SysSettings.h"

#ifdef VM_HEP_LIB_FULL
#include "VMHepLibTests.h"
#endif //#ifdef VM_HEP_LIB_FULL

////////////////////////////////////////////////////////////////////// includes END

#endif // !defined(VMLIB_H_INCLUDED_)
