////////////////////////////////////////////////////////
//	HEPEvtLiteUtils.cpp
//	Author: V.Makarenko (Vladimir.Makarenko@cern.ch)
////////////////////////////////////////////////////////

#include <iostream>
#include <fstream>
//#include <iomanip>

#include "VMLib/include/vmlib.h"

/*
*	--- CHEPEvtLiteLHE ---
*/
#define HEPEvtLiteLHE_StartTag		"LesHouchesEvents"
#define HEPEvtLiteLHE_HeaderTag		"header"
#define HEPEvtLiteLHE_InitTag		"init"
#define HEPEvtLiteLHE_EventTag		"event"

CHEPEvtLiteLHE::LHEInfo::LHEInfo(int _idIn1, int _idIn2,
								 tReal _EIn1, tReal _EIn2,
								 long _idPDF1, long _subPDF1, long _idPDF2, long _subPDF2,
								 int _idWgt, int _nProcs) {
	idIn[0]=_idIn1; idIn[1]=_idIn2; EIn[0]=_EIn1; EIn[1]=_EIn2;
	idPDF[0]=_idPDF1; idPDF[1]=_idPDF2; subPDF[0]=_subPDF1; subPDF[1]=_subPDF2; idWgt=_idWgt; nProcs=_nProcs;
	if(nProcs<0){ nProcs=0; }
}
CHEPEvtLiteLHE::LHEInfo::~LHEInfo() {
	std::vector< LHEProc* >::iterator i;
	for (i=procs.begin(); i!=procs.end(); ++i){ LHEProc* proc=*i; if(proc!=NULL)delete proc; }
}
void CHEPEvtLiteLHE::LHEInfo::AddProc(const CHEPEvtLiteLHE::LHEInfo::LHEProc& p) {
	procs.push_back(new LHEProc(p)); //don't search if process with same ID is already inserted
}
void CHEPEvtLiteLHE::LHEInfo::AddProc(const CHEPProcessLite* proc) {
	if(proc!=NULL){
		LHEProc p; p.iProc=proc->ProcID();
		tReal tmp; if(proc->GetCrossSection(tmp)) p.xSec=tmp;
		AddProc(p);
	}
}
const CHEPEvtLiteLHE::LHEInfo::LHEProc* CHEPEvtLiteLHE::LHEInfo::GetProcByID(int id) {
	std::vector< LHEProc* >::iterator i;
	for (i=procs.begin(); i!=procs.end(); ++i){ if(*i != NULL) { if((*i)->iProc == id) return (*i); } }
	return NULL;
}
const CHEPEvtLiteLHE::LHEInfo::LHEProc* CHEPEvtLiteLHE::LHEInfo::GetProc(int i) {
	return (((i>=0)&&(i<(int)procs.size()))?procs[i]:NULL);
}
//
CHEPEvtLiteLHE::CHEPEvtLiteLHE() {
	m_file_name=NULL;	//NULL if not ever open yet
	m_bReadMode=true;	//otherwise write
	m_info=NULL;
}
CHEPEvtLiteLHE::CHEPEvtLiteLHE(int _idIn1, int _idIn2,
					tReal _EIn1, tReal _EIn2,
					long _idPDF1, long _subPDF1, long _idPDF2, long _subPDF2,
					int _idWgt, int _nProcs) {
	m_file_name=NULL;	//NULL if not ever open yet
	m_bReadMode=true;	//otherwise write
	m_info=new LHEInfo(_idIn1,_idIn2,_EIn1,_EIn2,_idPDF1,_subPDF1,_idPDF2,_subPDF2,_idWgt,_nProcs);
}
CHEPEvtLiteLHE::~CHEPEvtLiteLHE() {
	char* tmp=m_file_name; m_file_name=NULL; CVMUtil::strFree(tmp);
	if(m_info!=NULL){ delete m_info; m_info=NULL; }
}
void CHEPEvtLiteLHE::SetProcessInfo(const CHEPProcessLite* proc) {
	if(m_info!=NULL) { m_info->AddProc(proc); }
}

// --- Write mode ---
bool CHEPEvtLiteLHE::StartLHEFile(const char* fn) {
	if(fn==NULL) return false;
	if(m_file_name!=NULL) { std::cout<<"[CHEPEvtLiteLHE] Can't open output file "<<fn<<" by current library"<<std::endl; return false; }
	const char* tab="  ";
	const char eol='\n';
	int i;

	std::ofstream ofs(fn,std::ios::out|std::ios::binary|std::ios::trunc);
	if(!ofs){ std::cout<<"[CHEPEvtLiteLHE] Can't open output file "<<fn<<std::endl; return false; }
	ofs<<"<"<<HEPEvtLiteLHE_StartTag<<" version=\"1.0\">"<<eol;

	ofs<<"<"<<HEPEvtLiteLHE_HeaderTag<<">"<<eol;
	ofs<<"</"<<HEPEvtLiteLHE_HeaderTag<<">"<<eol;

/*
<init>
  2212  2212  0.35000000000E+04  0.35000000000E+04 0 0 10042 10042 3  3
  0.10723674330E+03  0.11890260030E+00  0.27669000000E-02   0
  0.95759642100E+02  0.10617695111E+00  0.27669000000E-02   1
  0.73693614600E+02  0.81710448600E-01  0.27669000000E-02   2
</init>
*/
	ofs<<"<"<<HEPEvtLiteLHE_InitTag<<">"<<eol;
	if(m_info!=NULL) {
		//1. one line with process-number-independent information:
		//IDBMUP(1) IDBMUP(2) EBMUP(1) EBMUP(2) PDFGUP(1) PDFGUP(2) PDFSUP(1) PDFSUP(2) IDWTUP NPRUP
		ofs<<tab<<m_info->idIn[0]<<tab<<m_info->idIn[1]<<tab<<m_info->EIn[0]<<tab<<m_info->EIn[1];
		ofs<<tab<<m_info->idPDF[0]<<tab<<m_info->idPDF[1]<<tab<<m_info->subPDF[0]<<tab<<m_info->subPDF[1];
		ofs<<tab<<m_info->idWgt<<tab<<m_info->nProcs<<eol;

		//2. NPRUP lines, one for each process IPR in the range 1 through NPRUP:
		//XSECUP(IPR) XERRUP(IPR) XMAXUP(IPR) LPRUP(IPR)
		for(i=0; i<m_info->nProcs; i++) {
			const LHEInfo::LHEProc* proc=m_info->GetProc(i); if(proc==NULL)continue;
			if(proc->iProc < 0)continue;
			ofs<<tab<<proc->xSec<<tab<<proc->xSecErr<<tab<<proc->xMax<<tab<<proc->iProc<<eol;
		}
	}
	ofs<<"</"<<HEPEvtLiteLHE_InitTag<<">"<<eol;

	ofs.close();
	m_file_name=CVMUtil::strCpy(fn); m_bReadMode=false;
	return (m_file_name!=NULL);
}
bool CHEPEvtLiteLHE::SaveEvent(const CHEPEvtLite* evt) {
	if((m_bReadMode) || (m_file_name==NULL)) return false;
	std::ofstream ofs(m_file_name,std::ios::out|std::ios::binary|std::ios::app); if(!ofs){ return false; }
	const char eol='\n';
	const char* tab="  ";
/*
<event>
11   1  0.2766900E-02  0.1746972E+03  0.7546772E-02  0.1178469E+00
       21   -1    0    0  503  504  0.00000000000E+00  0.00000000000E+00  0.26318179454E+03  0.26318179454E+03  0.00000000000E+00 0. -1.
       21   -1    0    0  502  503  0.00000000000E+00  0.00000000000E+00 -0.24707340983E+03  0.24707340983E+03  0.00000000000E+00 0. -1.
        6    2    1    2  501    0  0.37146957165E+01  0.37867709408E+02  0.79416250994E+02  0.19367744655E+03  0.17250000000E+03 0. -1.
       -6    2    1    2    0  504  0.63950750766E+01 -0.16823902397E+02  0.65708236517E+02  0.18546633933E+03  0.17250000000E+03 0.  1.
       21    1    1    2  502  501 -0.10109770793E+02 -0.21043807011E+02 -0.12901610280E+03  0.13111141850E+03  0.00000000000E+00 0. -1.
        5    1    3    3  501    0  0.40313540039E+02  0.19735029221E+01  0.88629657656E+02  0.97505550819E+02  0.48000000000E+01 0. -1.
        2    1    3    3  505    0  0.17279691513E+02 -0.16606160593E+01 -0.23247599064E+02  0.29013725144E+02  0.00000000000E+00 0. -1.
       -1    1    3    3    0  505 -0.53878535836E+02  0.37554822545E+02  0.14034192403E+02  0.67158170587E+02  0.00000000000E+00 0.  1.
       -5    1    4    4    0  504 -0.50724165402E+02  0.32096026726E+02  0.40879829222E+02  0.72782527605E+02  0.48000000000E+01 0.  1.
        3    1    4    4  506    0  0.54121914528E+02 -0.67544203330E+02  0.72413061397E+01  0.86855267832E+02  0.00000000000E+00 0. -1.
       -4    1    4    4    0  506  0.29973259506E+01  0.18624274207E+02  0.17587101155E+02  0.25828543893E+02  0.14000000000E+01 0.  1.
#  0.1746972E+03  0.1746972E+03  0.2334629E+02
</event>
*/
	//
	ofs<<"<"<<HEPEvtLiteLHE_EventTag<<">"<<eol;

	//Event information, repeated as many times as there are events
	//a) one line with common event information:
	//  NUP IDPRUP XWGTUP SCALUP AQEDUP AQCDUP
	ofs<<evt->m_nParticles<<tab<<evt->idProc<<tab<<evt->wgt<<tab<<evt->scale<<tab<<evt->a_QED<<tab<<evt->a_QCD<<eol;

	//b) NUP lines, one for each particle I in the range 1 through NUP
	//  IDUP(I) ISTUP(I) MOTHUP(1,I) MOTHUP(2,I) ICOLUP(1,I) ICOLUP(2,I) PUP(1,I) PUP(2,I) PUP(3,I) PUP(4,I) PUP(5,I) VTIMUP(I) SPINUP(I)
	for(int i=0; i<evt->m_nParticles; i++) {
		const CHEPParticleLite* p = &(evt->m_Particles[i]);
		ofs<<tab<<p->m_PID<<tab<<p->m_Status<<tab<<(p->m_Mother1_f0+1)<<tab<<(p->m_Mother2_f0+1);
		int iCol1=0; int iCol2=0; //ICOLUP(1,I) ICOLUP(2,I)
		ofs<<tab<<iCol1<<tab<<iCol2;
		tReal mass = p->m_Mom.GetMassSigned(); if(mass<0)mass-=mass;
		ofs<<tab<<p->m_Mom.m_Px<<tab<<p->m_Mom.m_Py<<tab<<p->m_Mom.m_Pz<<tab<<p->m_Mom.m_E<<tab<<mass;
		tReal vtim = 0.; tReal spin = 0.; //VTIMUP(I) SPINUP(I)
		ofs<<tab<<vtim<<tab<<spin;
		ofs<<eol;
	}

	ofs<<"</"<<HEPEvtLiteLHE_EventTag<<">"<<eol;
	ofs.close();
	return true;
}
bool CHEPEvtLiteLHE::CloseLHEFile() {	//not required for 'read' mode
	bool bRes=true;
	const char eol='\n';
	if((!m_bReadMode) && (m_file_name!=NULL)) {
		//add closing tag
		std::ofstream ofs(m_file_name,std::ios::out|std::ios::binary|std::ios::app);
		if(ofs){ ofs<<"</"<<HEPEvtLiteLHE_StartTag<<">"<<eol; ofs.close(); }else{ bRes=false; }

		//re-init library
		char* tmp=m_file_name; m_file_name=NULL; CVMUtil::strFree(tmp); m_bReadMode=true;
	}
	return bRes;
}

// --- Read mode ---
bool CHEPEvtLiteLHE::OpenFile(const char* fn) { return false; }		//Not implemented
bool CHEPEvtLiteLHE::LoadEvent(CHEPEvtLite* evt) { return false; }	//Not implemented
